#include <daemon.h>
#include <fstp.h>
#include <zlib.h>

#define DAEMON_PORT 2233

fstp_pkt_t buf;
char *p=(char *)&buf;
char *str_seg[]={
	"Padding",
	"Packet data",
	"Command packet",
	"Timestamp",
	"EOT"
};

char *cmp_str[]={
	"No",
	"Zlib",
	"bzip2",
};

u_int32_t decompress_zpacket(p_fstp_pkt p, int level)
{
	int dlen=FSTP_MAX_PAYLOAD;
	char buf[FSTP_MAX_PAYLOAD];

	/* Copy to temporary buffer */
	memcpy((char *)&buf, (char *)&p->data, p->hdr.len-sizeof(p->hdr));
	
	/* Actually uncompress */
	uncompress((char *)&p->data, &dlen, (char *)&buf, p->hdr.len-sizeof(p->hdr));

	return dlen + sizeof(p->hdr);
}

int decode_packet(void *pkt, u_int32_t len)
{
	void *t=pkt;
	
	if ( !pkt || !len ) return 0;

	while( (t-pkt) <= len-1 ) {
		printf(" * %u byte segment. (Type: %s)\n",
			((p_segment_hdr)t)->caplen,
			((p_segment_hdr)t)->type < FSTP_SEG_MAX ?
			str_seg[((p_segment_hdr)t)->type] : "Unknown");
		
		t+=((p_segment_hdr)t)->caplen +
			sizeof(segment_hdr_t);
	}
}

int main ( int argc, char **argv )
{
	int lfd=-1;
	struct sockaddr_in svr;
	struct sockaddr_in from;
	int fromlen, rlen=0;

	printf("StormWall v1.0" CRLF COPYRIGHT_STRING);
	tzset();

	svr.sin_family=AF_INET;
	svr.sin_port=htons(2233);
	svr.sin_addr.s_addr=htonl(INADDR_LOOPBACK);
	
	if ( (lfd=socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0 ) {
		perror("socket() ");
		return EXIT_ERR;
	}

	if ( bind(lfd, (struct sockaddr *)&svr, sizeof(svr)) < 0 ) {
		perror("bind() ");
		close(lfd);
		return EXIT_ERR;
	}
	
	while (1){
		memset(p, 0, sizeof(buf));
		rlen=recvfrom(lfd, p, 1024, 0, (struct sockaddr *)&from, &fromlen);
		if ( rlen < 0 ) break;
		if ( rlen == 0 ) continue;
	
		printf("<= %s:%u ", 
			inet_ntoa(from.sin_addr),
			htons(from.sin_port));

		if ( rlen != buf.hdr.len ) {
			printf("ERROR: Length mismatch %i/%u" CRLF,
				rlen, buf.hdr.len);
			continue;
		}

		if ( buf.hdr.hdr_len != sizeof(buf.hdr) ) {
			printf("ERROR: Length mismatch %u (should be: %u)" CRLF,
				 buf.hdr.hdr_len, sizeof(buf.hdr));
			continue;
		}

		printf("[len=%i] %s Compression at level %u\n",
			buf.hdr.len,
			buf.hdr.compression < FSTP_COMPRESS_MAX ? 
				cmp_str[buf.hdr.compression] : 
				"Unknown",
			buf.hdr.level);
		
		switch ( buf.hdr.compression ) {
			case FSTP_COMPRESS_ZLIB:
				buf.hdr.len=decompress_zpacket(&buf, buf.hdr.level);
				break;
			case FSTP_COMPRESS_NONE:
			default:
		}
		decode_packet(p+sizeof(buf.hdr), buf.hdr.len-sizeof(buf.hdr));
		
		printf("\n");
	}
	
	perror("recvfrom() ");
	close(lfd);
	return EXIT_OK;
}
